unit gpMain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, ComCtrls,
  shellcommandrunner;

type

  { TGnuplotSettings }
  TGnuplotSettings = record
    PathToExe: string;
    OutputFileName: String;
    OutputFormat: String;
    OutputWidth: Integer;
    OutputHeight: Integer;
    ScriptFilename: String;
  end;

var
  GnuPlotSettings: TGnuplotSettings = (
//    PathToExe: 'd:\Programme\gnuplot\bin\gnuplot.exe';  // path to gnuplot.exe
    PathToExe: 'c:\Program Files (x86)\gnuplot\bin\gnuplot.exe';  // path to gnuplot.exe
    OutputFileName: '';                                 // output file of gnuplot
    OutputFormat: 'png';                                // gnuplot terminal for output to file
    OutputWidth: 400;                                   // width of gnuplot output picture
    OutputHeight: 400;                                  // height of gnuplot output picture
    ScriptFileName: '';                                 // filename of temporary script file
  );


type

  { TMainForm }

  TMainForm = class(TForm)
    BtnRunGnuplot: TButton;
    BtnKillGnuplot: TButton;
    PlotImage: TImage;
    ScriptMemo: TMemo;
    OutputMemo: TMemo;
    PageControl: TPageControl;
    Panel1: TPanel;
    PgScript: TTabSheet;
    PgOutput: TTabSheet;
    Splitter1: TSplitter;
    procedure BtnKillGnuplotClick(Sender: TObject);
    procedure BtnRunGnuplotClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
    FCmdThread: TShellCommandRunnerThread;
    FCmdThreadErrMsg: String;
    FCmdThreadAborted: Boolean;
    procedure CmdThreadOutputAvailable(const pBuffer: PByteArray; const pCount: Integer);
    procedure CmdThreadTerminate(Sender: TObject);
    function  DoExecGnuPlot(out ErrMsg:string) : boolean;
  public
    { public declarations }
  end;

var
  MainForm: TMainForm;


implementation

{$R *.lfm}
uses
  Process;

{ Utilities }

function UseUnixPathdelimiters(APath:string) : string;
// Gnuplot wants "/" als path delimiter, no "\"!
var
  i : integer;
begin
  result := APath;
  for i:=1 to Length(result) do
    if result[i] = '\' then
      result[i] := '/'
end;


{ TMainForm }

procedure TMainForm.BtnKillGnuplotClick(Sender: TObject);
begin
  if FCmdThread <> nil then
    FCmdThread.Abort;
  BtnKillGnuplot.Visible := false;
end;

procedure TMainForm.BtnRunGnuplotClick(Sender: TObject);
var
  ok: Boolean;
  err: String;
begin
  ok := DoExecGnuplot(err);
  PgOutput.TabVisible := not ok;
  if not ok then begin
    OutputMemo.Lines.Text := err;
    pageControl.ActivePage := PgOutput;
  end;
end;

procedure TMainForm.CmdThreadOutputAvailable(const pBuffer: PByteArray;
  const pCount: Integer);
begin
  FCmdThreadErrMsg := FCmdThreadErrMsg + TShellCommandRunner.BufferToString(pBuffer,pCount)
end;

procedure TMainForm.CmdThreadTerminate(Sender: TObject);
begin
  if (FCmdThread.Runner.ExitStatus = 259) then begin
    FCmdThreadErrMsg := '';
    FCmdThreadAborted := true;
    // gnuplot is still running, the errmsg is the gnuplot prompt in this case
  end;

  if (FCmdThread.Runner.ExitStatus = 0) and (FCmdThreadErrMsg = '') then
    PlotImage.Picture.LoadFromFile(GnuplotSettings.OutputFilename)
  else
    PlotImage.Picture.Clear;
  BtnKillGnuPlot.Visible := false;
  FCmdThread := nil;
end;

function TMainForm.DoExecGnuPlot(out ErrMsg: string): boolean;
var
  List: TStringList;
  crs: TCursor;
  res: integer;
begin
  result := false;
  ErrMsg := '';

  if (GnuPlotSettings.PathToExe = '') then begin
    ErrMsg := 'Path to gnuplot binary is not specified. Please use settings dialog.';
    exit;
  end;
  if not FileExists(GnuPlotSettings.PathToExe) then begin
    ErrMsg := Format('Gnuplot binary is not found, file "%s" does not exist. '+
      'Please use settings dialog.', [GnuPlotSettings.PathToExe]);
    exit;
  end;

  if FCmdThread <> nil then begin
    ErrMsg := 'Gnuplot still running.';
    exit;
  end;

  crs := Screen.Cursor;
  Screen.Cursor := crHourglass;

  try
    ScriptMemo.Lines.SaveToFile(GnuplotSettings.ScriptFileName);

    BtnKillGnuplot.Visible := true;

    FCmdThreadErrMsg := '';
    FCmdThreadAborted := false;
    FCmdThread := TShellCommandRunnerThread.Create;
    // Note: the thread has FreeOnTerminate=true --> no Free here.
    // Also, see OnTerminate event handler for further actions.
    with FCmdThread do begin
      OnOutputAvailable := @CmdThreadOutputAvailable;
      OnTerminate := @CmdThreadTerminate;
      CommandLine := Format(
        '"%s" -e "set terminal %s size %d,%d truecolor enhanced" -e "set output ''%s''" "%s"',
        [
          GnuplotSettings.PathToExe,
          GnuplotSettings.OutputFormat,
          GnuplotSettings.OutputWidth,
          GnuplotSettings.OutputHeight,
          UseUnixPathDelimiters(GnuplotSettings.OutputFileName),
          GnuplotSettings.ScriptFileName
        ]
      );
      Start;
    end;

    // this is a replacement for "WaitFor" -- system remains responsive.
    while (FCmdThread <> nil) do
      Application.ProcessMessages;

    ErrMsg := FCmdThreadErrMsg;
    result := (ErrMsg = '') and (not FCmdThreadAborted);
  finally
    Screen.Cursor := crs;
  end;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  if Assigned(FCmdThread) then
    FCmdThread.Abort;
  if FileExists(GnuplotSettings.ScriptFilename) then
    DeleteFile(GnuplotSettings.ScriptFileName);
  if FileExists(GnuplotSettings.OutputFileName) then
    DeleteFIle(GnuplotSettings.OutputFileName);
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  tmp: String;
begin
  tmp := IncludeTrailingPathDelimiter(SysUtils.GetTempDir);
  GnuplotSettings.ScriptFileName := tmp + 'script.plt';
  GnuplotSettings.OutputFilename := tmp + 'output.png';
  pageControl.ActivePage := PgScript;
end;

end.

